#!/bin/ksh
#\
exec wish "$0" "$@"

set defaultfields { 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 0}
set fields $defaultfields
set previousfields $defaultfields

set fieldlabels {
	"Comment"
	"Last Name"
	"First Name"
	"Company"
	"Title"
	"Work Address"
	"Suite/P.O.Box"
	"Work City"
	"Work State"
	"Work Zip Code"
	"Work Country"
	"Home Address"
	"Appt/P.O.Box"
	"Home City"
	"Home State"
	"Home Zip Code"
	"Home Country"
	"Work Phone"
	"Home Phone"
	"Main Phone"
	"Work Fax"
	"Home Fax"
	"Mobile Phone"
	"Email"
	"WWW Address"
}

proc new_ccat {w} {
global contactscatlist ccat
	grab $w
	set lcat [string trim [$w.add.e get]]
	if {$lcat!={}} {
		lappend contactscatlist $lcat
		$w.c.c.m.menu add radiobutton -variable ccat -label "$lcat"
	}
	destroy $w.add
}

proc add_ccat {w} {
global contactscatlist
global entrybgcolor entryfgcolor entryfont
set max 4
	if {[llength $contactscatlist] >= $max} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "Only $max categories allowed"
		return
	}
	toplevel $w.add
	wm title $w.add "Add Contacts Category"
	entry $w.add.e -fg $entryfgcolor -bg $entrybgcolor -font $entryfont
	bind $w.add.e <Return> "new_ccat {$w}"
	pack $w.add.e -expand yes -fill x
	focus $w.add.e
	grab $w.add
}

proc del_ccat {w} {
global contactscatlist ccat

	if {$ccat=="All"} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "Can't delete All category"
		return
	}

	if {[tk_messageBox -icon question -type yesno -title "Are You Sure ?" \
		-message "Are You Sure"]=="no"} {
		return
	}

	set n 0
	foreach i $contactscatlist {
		if {$i==$ccat} {
			set contactscatlist [lreplace $contactscatlist $n $n]
			$w.c.c.m.menu delete $n
			break
		}
		incr n
	}
	set ccat All
}

proc num_to_cat {i} {
	global contactscatlist
	return [lindex $contactscatlist [expr $i - 1]]
}

proc cat_to_num {cat} {
	global contactscatlist
	set catn 0
	foreach i $contactscatlist {
		if {$i==$cat} {
			incr catn
			break
		}
		incr catn
	}
	return $catn
}

proc fill_contacttext {w} {
set win $w.c.details
global sel

	set var [$w.c.nb raised]
	set sel [$w.c.nb.nbframe.$var.lb cursel]
	if {$sel=={}} return
	global $var
	set i [lindex [set $var] $sel]
	global contactdetails$i ccat
	array set cd [array get contactdetails$i]

	entry_state $w normal
	clear_entries $w
	foreach j [array names cd] {
		if [winfo exists $win.frame$j.e] {
			instext $win.frame$j.e $cd($j)
		}
	}
	set ccat [num_to_cat $cd(ccat)]
	entry_state $w disabled
}

proc clear_entries {w} {
	foreach win [winfo children $w.c.details] {
		if [winfo exists $win.e] {
			instext $win.e {}
		}
	}
}

proc clear_contact {w} {
	foreach win $w.c.nb.nbframe.[$w.c.nb raised] {
		$win.lb selection clear 0 end
	}
	clear_entries $w
}

proc entry_state {w state} {
	set topwin {}
	foreach win [winfo children $w.c.details] {
		if [winfo exists $win.e] {
			if {$topwin=={} && $state=="normal"} {
				set topwin $win.e
				focus $topwin
			}
			$win.e config -state $state
		}
	}
}

proc restore_addressbook {w} {
	global sel
	set sel -1
	entry_state $w disabled
	pack forget $w.buttons.s $w.buttons.dnc
	pack $w.c.nb -fill both -expand yes -side left
	pack $w.buttons.n -side left -expand yes -fill x -pady 2
	pack $w.buttons.e -side left -expand yes -fill x -pady 2
	pack $w.buttons.q -side right -expand yes -fill x -pady 2
	pack $w.buttons.d -side right -expand yes -fill x -pady 2
}

proc hide_addressbook {w} {
	pack forget $w.c.nb $w.buttons.n $w.buttons.e $w.buttons.d $w.buttons.q
	pack $w.buttons.s -side left -expand yes -fill x -pady 2
	pack $w.buttons.dnc -side right -expand yes -fill x -pady 2
	entry_state $w normal
}

proc edit_contact {w} {
	set var [$w.c.nb raised]
	set cursel [$w.c.nb.nbframe.$var.lb cursel]
	if {$cursel=={}} return
	global editing
	set editing $cursel
	hide_addressbook $w
	$w.buttons.s config -text Done -command "save_edited_contact {$w}"
}

proc new_contact {w} {
	clear_contact $w
	hide_addressbook $w
	$w.buttons.s config -text Save -command "save_new_contact {$w}"
}

proc pc {i} {
	global contactdetails$i contactlist$i
	puts "-: [set contactlist$i] [array get contactdetails$i]"
}
proc save_new_contact {w} {
	restore_addressbook $w

	if {[string trim [$w.c.details.frame1.e get]]=={} ||
	    [string trim [$w.c.details.frame2.e get]]=={}} {
			tk_messageBox -icon info -type ok -title Error \
			-message "Critical fields empty.\nNot saving entry.\nFill in First/Last name."
			clear_contact $w
			return
	}

	global fields ncontacts ccat
	set i [incr ncontacts]
	global contactdetails$i contactlist$i
	lappend contactlist$i [list contact [cat_to_num $ccat] 0]
	array set contactdetails$i [list ccat [cat_to_num $ccat]]
	foreach n $fields {
		set text [string trim [$w.c.details.frame$n.e get]]
		if {$text=={}} continue
		lappend contactlist$i [list $n 0 "$text"]
		array set contactdetails$i [list $n $text]
	}
	set var [add_to_contactlist $w $i]
	$w.c.nb raise $var
#	puts [set contactlist$i]
}

proc save_edited_contact {w} {
	restore_addressbook $w
	set var [$w.c.nb raised]
	global editing
	if [info exists editing] {
			set cursel $editing
			unset editing
#			puts $cursel
	}
	if {[string trim [$w.c.details.frame1.e get]]=={} ||
	    [string trim [$w.c.details.frame2.e get]]=={}} {
			tk_messageBox -icon info -type ok -title Error \
			-message "Critical fields empty.\nNot saving entry.\nFill in First/Last name."
			clear_contact $w
			return
	}
	if {$cursel=={}} {
		tk_messageBox -icon info -type ok -title Error -message \
		"Something went wrong...\nNot updating !"
		return
	}
	global $var
	set $var [lreplace [set $var] $cursel $cursel]
	set win $w.c.nb.nbframe.$var.lb
	$win del $cursel

	global fields ncontacts ccat
	set i [incr ncontacts]
	global contactdetails$i contactlist$i
	lappend contactlist$i [list contact [cat_to_num $ccat] 0]
	array set contactdetails$i [list ccat [cat_to_num $ccat]]
	foreach n $fields {
		set text [string trim [$w.c.details.frame$n.e get]]
		if {$text=={}} continue
		lappend contactlist$i [list $n 0 "$text"]
		array set contactdetails$i [list $n $text]
	}
	set var [add_to_contactlist $w $i]
	$w.c.nb raise $var
#	puts [set contactlist$i]
}

proc delete_contact {w} {
global ncontacts ccat

	# Find out which tabed widget window is raised
	set var [$w.c.nb raised]
	# Find the selection index in the listbox
	set cursel [$w.c.nb.nbframe.$var.lb cursel]

	# Some sanity checking
	if {$cursel=={}} {
		tk_messageBox -icon info -type ok -title Error -message \
		"No Selection"
		return
	}
	if {[tk_messageBox -icon question -type yesno -title "Are You Sure ?" \
		-message "Are You Sure"]=="no"} {
		return
	}

	global $var
	# Find the index in the tabed variable list
	# that corresponds to the global var
	set i [lindex [set $var] $cursel]

	# Remove the var from the tabed variable list
	set $var [lreplace [set $var] $cursel $cursel]
	$w.c.nb.nbframe.$var.lb del $cursel

	# Remove the variables for this entry
	global contactlist$i contactdetails$i
	unset contactlist$i contactdetails$i

	# Reset the category to the base default
	set ccat All
}

proc swap_fields {} {
	global fields previousfields
	set n "$fields"

	set fields $previousfields
	set previousfields $n
}

proc change_label {w i} {
	global entrybgcolor entryfgcolor entryfont

	toplevel $w.add
	wm title $w.add "Change Label"
	entry $w.add.e -fg $entryfgcolor -bg $entrybgcolor -font $entryfont
	bind $w.add.e <Return> "destroy_change_label $w $i"
	instext $w.add.e "[$w.c.details.frame$i.l cget -text]"
	$w.add.e config -state normal
	pack $w.add.e -expand yes -fill x
	focus $w.add.e
	grab $w.add
}
proc destroy_change_label {w i} {
	grab $w
	global fieldlabels

	set label [string trim [$w.add.e get]]
	if {$label!={}} {
		$w.c.details.frame$i.l config -text $label
		set fieldlabels [lreplace $fieldlabels $i $i $label]
	}
	destroy $w.add
}

proc popup_menu {w i} {
	global fields previousfields defaultfields
	global labelbgcolor labelfgcolor labelfont
	global menufont
	set m $w.c.details.frame$i

	if {[winfo exists $m.menu]} {destroy $m.menu}
	menu $m.menu -tearoff 0 -fg $labelfgcolor -bg $labelbgcolor
	$m.menu add command -font $menufont -label "Change field label" \
		-command "change_label $w $i"
	$m.menu add command -font $menufont -label "Remove this field" \
		-command "
			set previousfields {$fields}
			remove_fromcontactlist {$w} $i
		"
	$m.menu add command -font $menufont -label "Unhide all entries" \
		-command "
			if {\"$fields\"==\"$defaultfields\"} return
			set previousfields {$fields}
			set fields {$defaultfields}
			build_contactswin {$w}
			fill_contacttext {$w}
		"
	$m.menu add command -font $menufont -label "Toggle last fields" \
		-command "
			swap_fields
			build_contactswin {$w}
			fill_contacttext {$w}
		"
	$m.menu add command -font $menufont -label "Save this layout" \
		-command "save_user_settings fields \"$fields\""

	tk_popup $m.menu [winfo pointerx $m] [winfo pointery $m]
}

proc remove_fromcontactlist {w i} {
	global fields

	if {[llength $fields]==1} {
		tk_messageBox -icon error -type ok -title Error -message \
		"Can't remove the last entry."
		return
	}

	if {$i==1 || $i==2} {
		tk_messageBox -icon error -type ok -title Error -message \
		"Can't remove critical fields."
		return
	}

	set m $w.c.details.frame$i
	set index [lsearch $fields $i]

	if {$index >= 0} {
		set fields [lreplace $fields $index $index]
		destroy $w.c.details.frame$i
}
}

proc _contacts {w} {
global labelbgcolor labelfgcolor labelfont
global buttonbgcolor buttonfgcolor buttonfont
global ncontacts contactscatlist ccat

if [winfo exists $w] return
toplevel $w
wm title $w "Contacts Editor"
wm iconname $w "Contacts Editor"
grab $w

# Break the contacts into arrays
for {set i 0} {$i<=$ncontacts} {incr i} {
	global contactlist$i
	foreach x [set contactlist$i] {
		global contactdetails$i
		switch [lindex $x 0] {
			contact {
				array set contactdetails$i \
					[list ccat [lindex $x 1]]
			}
			default {
				array set contactdetails$i \
					[list [lindex $x 0] [lindex $x 2]]
			}
		}
	}
}

frame $w.c -bg $labelbgcolor -bd 5

frame $w.c.c -bg $labelbgcolor -bd 2 -re sunken
label $w.c.c.l -text Category \
	-fg $labelfgcolor -bg $labelbgcolor -font $labelfont -width 10
set m [tk_optionMenu $w.c.c.m ccat {}]
$m configure -fg $labelfgcolor -bg $labelbgcolor -font $labelfont
$w.c.c.m configure -fg $labelfgcolor -bg $labelbgcolor -font $labelfont \
	-relief flat -bd 0 -activeback $labelbgcolor -padx 0 -pady 2 \
	-highlightthickness 0
$m delete 0
foreach i $contactscatlist {
	$m add radiobutton -variable ccat -label $i
}
set ccat All
button $w.c.c.a -text Add... -command "add_ccat {$w}" \
	-fg $labelfgcolor -bg $labelbgcolor -font $labelfont \
	-bd 1
button $w.c.c.d -text Delete -command "del_ccat {$w}" \
	-fg $labelfgcolor -bg $labelbgcolor -font $labelfont \
	-bd 1
pack $w.c.c.l -side left
pack $w.c.c.m -side left -expand yes -fill x
pack $w.c.c.d -side right -anchor e
pack $w.c.c.a -side right -anchor e

tixNoteBook $w.c.nb
$w.c.nb subwidget nbframe config \
	-tabpadx 10 -tabpady 0 \
	-backpagecolor $labelbgcolor \
	-font $labelfont -bg $labelbgcolor -fg $labelfgcolor

set win $w.c.nb
$win add a-e -label "A-E" -raisecmd "clear_contact {$w}"
$win add f-j -label "F-J" -raisecmd "clear_contact {$w}"
$win add k-o -label "K-O" -raisecmd "clear_contact {$w}"
$win add p-t -label "P-T" -raisecmd "clear_contact {$w}"
$win add u-z -label "U-Z" -raisecmd "clear_contact {$w}"
$win add misc -label "Misc" -raisecmd "clear_contact {$w}"

foreach win [winfo children $w.c.nb.nbframe] {
	$win config -bg $labelbgcolor
	listbox $win.lb -font $labelfont -selectmode single -width 40
	pack $win.lb -expand yes -fill both -padx 5 -pady 5
}

for {set i 0} {$i<=$ncontacts} {incr i} {
	add_to_contactlist $w $i
}

frame $w.c.details -bg $labelbgcolor -bd 4
build_contactswin $w

frame $w.buttons -bg $labelbgcolor -bd 2 -re ridge

button $w.buttons.n -text New \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1 \
	-command "new_contact {$w}"
button $w.buttons.e -text Edit \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1 \
	-command "edit_contact {$w}"
button $w.buttons.s -text Save \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
button $w.buttons.d -text Delete \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1 \
	-command "delete_contact {$w}"
button $w.buttons.q -text Dismiss \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1 \
	-command "destroy $w"
button $w.buttons.dnc -text Cancel \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1 \
	-command "restore_addressbook {$w}"

pack $w.buttons.n -side left -expand yes -fill x -pady 2
pack $w.buttons.e -side left -expand yes -fill x -pady 2
pack $w.buttons.q -side right -expand yes -fill x -pady 2
pack $w.buttons.d -side right -expand yes -fill x -pady 2

pack $w.c.c -side bottom -fill x -expand yes -pady 2
pack $w.c.nb -side left -fill both -expand yes
pack $w.c.details -side right -expand yes -fill both
pack $w.c -expand yes -fill both
pack $w.buttons -fill x
}

proc build_contactswin {w} {
global labelbgcolor labelfgcolor labelfont entrybgcolor entryfgcolor entryfont
global fields fieldlabels

foreach win [winfo children $w.c.details] {destroy $win}
set win $w.c.details
foreach i "$fields" {

	set text [lindex $fieldlabels $i]

	if {[winfo exists $w.frame$i]} {continue}

	frame $win.frame$i -bg $labelbgcolor

	label $win.frame$i.l -font $labelfont \
		-font $labelfont -fg $labelfgcolor -bg $labelbgcolor \
		-bd 0 -relief flat -text $text -justify left
	entry $win.frame$i.e -relief sunken -width 20 \
		-font $entryfont -fg $entryfgcolor -bg $entrybgcolor

	bind $win.frame$i.l <Double-Button-1> "popup_menu {$w} $i"

	pack $win.frame$i $win.frame$i.l -side left -anchor w
	pack $win.frame$i $win.frame$i.e -side right -anchor e
	pack $win.frame$i -side top -fill both
}
}

proc add_to_contactlist {w i} {
global contactdetails$i
global a-e f-j k-o p-t u-z misc
set win $w.c.nb.nbframe

	array set cd [array get contactdetails$i]
	if [array names cd 1] {set surname [string trim $cd(1)]}
	if [array names cd 2] {set firstname [string trim $cd(2)]}

	switch -regexp $surname {
		^[A-E]+|^[a-e]+	{set lbwin a-e}
		^[F-J]+|^[f-j]+	{set lbwin f-j}
		^[K-O]+|^[k-o]+	{set lbwin k-o}
		^[P-T]+|^[p-t]+	{set lbwin p-t}
		^[U-Z]+|^[u-z]+	{set lbwin u-z}
		default		{set lbwin misc}
	}
	lappend $lbwin $i
	$win.$lbwin.lb insert end "$surname, $firstname"
	bind $win.$lbwin.lb <ButtonRelease> "fill_contacttext {$w}"
	return $lbwin
}
