#!/bin/ksh
#\
exec wish "$0" "$@"

proc new_mcat {w} {
global memoscatlist mcat
	grab $w
	set lcat [string trim [$w.add.e get]]
	if {$lcat!={}} {
		lappend memoscatlist $lcat
		$w.f1.m.menu add radiobutton -variable mcat -label "$lcat"
	}
	destroy $w.add
}

proc add_mcat {w} {
global memoscatlist
global entrybgcolor entryfgcolor entryfont
set max 5

	if {[llength $memoscatlist] >= $max} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "Only $max categories allowed"
		return
	}
	toplevel $w.add
	wm title $w.add "Add Memo Category"
	entry $w.add.e -fg $entryfgcolor -bg $entrybgcolor -font $entryfont
	bind $w.add.e <Return> "new_mcat {$w}"
	pack $w.add.e -expand yes -fill x
	focus $w.add.e
	grab $w.add
}

proc del_mcat {w} {
global memoscatlist mcat

	if {$mcat=="All"} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "Can't delete All category"
		return
	}

	if {[tk_messageBox -icon question -type yesno -title "Are You Sure ?" \
		-message "Are You Sure"]=="no"} {
		return
	}

	set n 0
	foreach i $memoscatlist {
		if {$i==$mcat} {
			set memoscatlist [lreplace $memoscatlist $n $n]
			$w.f1.m.menu delete $n
			break
		}
		incr n
	}
	set mcat All
}

proc show_memo {w} {
set i [$w.f3.lb curselection]
global memolist$i
global memoscatlist mcat

	if {$i=={}} return

	instext $w.f2.e [lindex [set memolist$i] 2]
	set catn [lindex [set memolist$i] 1]
	incr catn -1
	if {$catn<0} {
		puts "Fatal Error - category is 0 ! How can that be ?"
		exit 1
	}
	set mcat [lindex $memoscatlist $catn]

	$w.f4.t conf -state normal
	$w.f4.t delete 0.0 end
	$w.f4.t insert 0.0 [lindex [set memolist$i] 3]
	$w.f4.t conf -state disabled
}

proc edit_memo {w} {
	set i [$w.f3.lb curselection]
	if {$i=={}} {
		tk_messageBox -icon error -type ok -title Error \
			-message "No Selection"
		return
	}
	$w.f2.e conf -state normal
	$w.f4.t conf -state normal
	focus $w.f4.t
}

proc new_memo {w} {
	$w.f2.e conf -state normal
	$w.f4.t conf -state normal
	$w.f2.e delete 0 end
	$w.f4.t delete 0.0 end
	focus $w.f2.e
}

proc save_memo {w} {
global memoscatlist mcat
global nmemos

	set title [string trim [$w.f2.e get]]
	if {$title=={}} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "No title !"
		return
	}
	set x [$w.f3.lb curselection]

	if {$x!={}} {
		$w.f3.lb delete $x
		$w.f3.lb insert $x "$title"
	} else {
		$w.f3.lb insert end "$title"
	}

	set catn 0
	foreach i $memoscatlist {
		if {$i==$mcat} {
			incr catn
			break
		}
		incr catn
	}
	if {$x=={}} {
		set x [incr nmemos]
	}
	global memolist$x
	set memolist$x "memo $catn \"$title\" \"[convstr [$w.f4.t get 0.0 end]]\""
	$w.f2.e conf -state disabled
	$w.f4.t conf -state disabled
}

proc del_memo {w} {
global nmemos memolist

	set i [$w.f3.lb curselection]
	$w.f3.lb select clear 0 end

	if {$i=={}} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "No Selection"
		return
	}

	for {set x $i; set y [expr $x + 1]} {$x < $nmemos} {incr x; incr y} {
		global memolist$x memolist$y
		set memolist$x [set memolist$y]
	}

	global memolist$nmemos
	unset memolist$nmemos
	incr nmemos -1

	$w.f3.lb delete $i

	$w.f2.e conf -state normal
	$w.f4.t conf -state normal
	$w.f2.e delete 0 end
	$w.f4.t delete 0.0 end
	$w.f2.e conf -state disabled
	$w.f4.t conf -state disabled
}

proc _memo {w} {
global labelbgcolor labelfgcolor labelfont
global buttonbgcolor buttonfgcolor buttonfont
global entrybgcolor entryfgcolor entryfont
global memoscatlist mcat
global nmemos

if [winfo exists $w] return
toplevel $w
wm title $w "Memo Editor"
wm iconname $w "Memo Editor"
grab $w

$w config -bg $labelbgcolor

frame $w.f1 -bg $labelbgcolor
label $w.f1.l -text Category \
	-fg $labelfgcolor -bg $labelbgcolor -font $labelfont -width 10
set m [tk_optionMenu $w.f1.m mcat {}]
$m configure -fg $labelfgcolor -bg $labelbgcolor -font $labelfont
$w.f1.m configure -fg $labelfgcolor -bg $labelbgcolor -font $labelfont \
	-relief flat -bd 0 -activeback $labelbgcolor -padx 0 -pady 2 \
	-highlightthickness 0
$m delete 0
foreach i $memoscatlist {
	$m add radiobutton -variable mcat -label $i
}
set mcat All
button $w.f1.a -text Add... -command "add_mcat {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont \
	-bd 1
button $w.f1.d -text Delete -command "del_mcat {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont \
	-bd 1

pack $w.f1.l -side left
pack $w.f1.m -side left -expand yes -fill x
pack $w.f1.d -side right -anchor e
pack $w.f1.a -side right -anchor e

frame $w.f2 -bg $labelbgcolor
label $w.f2.l -text Title \
	-fg $labelfgcolor -bg $labelbgcolor -font $labelfont -width 10
entry $w.f2.e -fg $entryfgcolor -bg $entrybgcolor -font $entryfont -state disabled
pack $w.f2.l -side left
pack $w.f2.e -side left -expand yes -fill x

frame $w.f3 -bg $labelbgcolor
scrollbar $w.f3.lsb -command "$w.f3.lb yview" -bg $labelbgcolor -width 10
listbox $w.f3.lb -font $labelfont -selectmode single -width 80\
	-yscroll "$w.f3.lsb set" -setgrid 1
bind $w.f3.lb <ButtonRelease> "show_memo {$w}"
pack $w.f3.lb -expand yes -fill both -side left -anchor w
pack $w.f3.lsb -expand yes -fill y -side right -anchor e

frame $w.f4 -bg $labelbgcolor
scrollbar $w.f4.tsb -command "$w.f4.t yview" -width 10 -bg $labelbgcolor
text $w.f4.t -state disabled \
	-fg $entryfgcolor -bg $entrybgcolor -font $entryfont -width 80\
	-yscroll "$w.f4.tsb set"
for {set i 0} {$i <= $nmemos} {incr i} {
	global memolist$i
	$w.f3.lb insert end [lindex [set memolist$i] 2]
}
pack $w.f4.t -expand yes -fill both -side left -anchor w
pack $w.f4.tsb -expand yes -fill y -side right -anchor e

frame $w.f5 -bg $labelbgcolor -bd 2 -re ridge
button $w.f5.e -text Edit -command "edit_memo {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
button $w.f5.n -text New -command "new_memo {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
button $w.f5.s -text Save -command "save_memo {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
button $w.f5.d -text Delete -command "del_memo {$w}" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
button $w.f5.q -text Dismiss -command "destroy $w" \
	-fg $buttonfgcolor -bg $buttonbgcolor -font $buttonfont -bd 1
pack $w.f5.e -side left -expand yes -fill x
pack $w.f5.n -side left -expand yes -fill x
pack $w.f5.s -side left -expand yes -fill x
pack $w.f5.d -side left -expand yes -fill x
pack $w.f5.q -side left -expand yes -fill x

pack $w.f1 -expand yes -fill x -padx 10 -pady 2
pack $w.f2 -expand yes -fill x -padx 10 -pady 2
pack $w.f3 -expand yes -fill x -padx 10 -pady 10
pack $w.f4 -expand yes -fill x -padx 10 -pady 10
pack $w.f5 -expand yes -fill x
}
